@@ -1,6 +1,6 @@
 package Math::Vector::Real;
 
-our $VERSION = '0.08';
+our $VERSION = '0.11';
 
 use strict;
 use warnings;
@@ -10,6 +10,12 @@ use POSIX ();
 use Exporter qw(import);
 our @EXPORT = qw(V);
 
+local ($@, $!, $SIG{__DIE__});
+
+our $dont_use_XS;
+
+eval { require Math::Vector::Real::XS } unless $dont_use_XS;
+
 our %op = (add => '+',
 	   neg => 'neg',
 	   sub => '-',
@@ -87,6 +93,7 @@ sub _caller_op {
 }
 
 sub _check_dim {
+    local ($@, $SIG{__DIE__});
     eval { @{$_[0]} == @{$_[1]} } and return;
     my $op = _caller_op(1);
     my $loc = ($_[2] ? 'first' : 'second');
@@ -145,18 +152,10 @@ sub mul {
 }
 
 sub mul_me {
-    if (ref $_[1]) {
-	&_check_dim;
-	my ($v0, $v1) = @_;
-	my $acu = 0;
-	$acu += $v0->[$_] * $v1->[$_] for 0..$#$v0;
-	$acu;
-    }
-    else {
-	my ($v, $s) = @_;
-	$_ *= $s for @$v;
-	$v
-    }
+    ref $_[1] and croak "can not multiply by a vector in place as the result is not a vector";
+    my ($v, $s) = @_;
+    $_ *= $s for @$v;
+    $v
 }
 
 sub div {
@@ -215,12 +214,16 @@ sub abs {
     sqrt $acu;
 }
 
+*norm = \&abs;
+
 sub abs2 {
     my $acu = 0;
     $acu += $_ * $_ for @{$_[0]};
     $acu;
 }
 
+*norm2 = \&abs2;
+
 sub dist {
     &_check_dim;
     my ($v0, $v1) = @_;
@@ -243,6 +246,20 @@ sub dist2 {
     $d2;
 }
 
+sub manhattan_norm {
+    my $n = 0;
+    $n += CORE::abs($_) for @{$_[0]};
+    return $n;
+}
+
+sub manhattan_dist {
+    &_check_dim;
+    my ($v0, $v1) = @_;
+    my $d = 0;
+    $d += CORE::abs($v0->[$_] - $v1->[$_]) for 0..$#$v0;
+    return $d;
+}
+
 sub _upgrade {
     my $dim;
     map {
@@ -260,11 +277,18 @@ sub _upgrade {
 
 sub atan2 {
     my ($v0, $v1) = @_;
-    my $a0 = &abs($v0);
-    return 0 unless $a0;
-    my $u0 = $v0 / $a0;
-    my $p = $v1 * $u0;
-    CORE::atan2(&abs($v1 - $p * $u0), $p);
+    if (@$v0 == 2) {
+        my $dot = $v0->[0] * $v1->[0] + $v0->[1] * $v1->[1];
+        my $cross = $v0->[0] * $v1->[1] - $v0->[1] * $v1->[0];
+        return CORE::atan2($cross, $dot);
+    }
+    else {
+        my $a0 = &abs($v0);
+        return 0 unless $a0;
+        my $u0 = $v0 / $a0;
+        my $p = $v1 * $u0;
+        CORE::atan2(&abs($v1 - $p * $u0), $p);
+    }
 }
 
 sub versor {
@@ -285,7 +309,7 @@ sub wrap {
 		  $c - $s * POSIX::floor($c/$s) } (0..$#$self)];
 }
 
-sub max {
+sub max_component {
     my $max = 0;
     for (@{shift()}) {
 	my $abs = CORE::abs($_);
@@ -294,7 +318,7 @@ sub max {
     $max
 }
 
-sub min {
+sub min_component {
     my $self = shift; 
     my $min = CORE::abs($self->[0]);
     for (@$self) {
@@ -304,12 +328,20 @@ sub min {
     $min
 }
 
+*max = \&max_component;
+*min = \&min_component;
+
+sub first_orthant_reflection {
+    my $self = shift;
+    bless [map CORE::abs, @$self];
+}
+
 sub box {
     shift;
     return unless @_;
     my $min = clone(shift);
     my $max = clone($min);
-    my $dim = @$min - 1;
+    my $dim = $#$min;
     for (@_) {
         for my $ix (0..$dim) {
             my $c = $_->[$ix];
@@ -321,23 +353,86 @@ sub box {
             }
         }
     }
-    ($min, $max);
+    wantarray ? ($min, $max) : $max - $min;
+}
+
+sub nearest_in_box {
+    my $p = shift->clone;
+    my ($min, $max) = Math::Vector::Real->box(@_);
+    for (0..$#$p) {
+        if ($p->[$_] < $min->[$_]) {
+            $p->[$_] = $min->[$_];
+        }
+        elsif ($p->[$_] > $max->[$_]) {
+            $p->[$_] = $max->[$_];
+        }
+    }
+    $p
+}
+
+sub nearest_in_box_border {
+    my $p = shift->clone;
+    my ($b0, $b1) = Math::Vector::Real->box(@_);
+    my ($min_d, $comp, $comp_ix);
+    for my $q ($b0, $b1) {
+        for (0..$#$p) {
+            my $d = CORE::abs($p->[$_] - $q->[$_]);
+            if (!defined $min_d or $min_d > $d) {
+                $min_d = $d;
+                $comp = $q->[$_];
+                $comp_ix = $_;
+            }
+        }
+    }
+    $p->[$comp_ix] = $comp;
+    wantarray ? ($p, $min_d) : $p;
+}
+
+sub max_dist2_between_boxes {
+    my ($class, $a0, $a1, $b0, $b1) = @_;
+    my ($c0, $c1) = $class->box($a0, $a1);
+    my ($d0, $d1) = $class->box($b0, $b1);
+    my $d2 = 0;
+    for (0..$#$c0) {
+        my $e0 = $d1->[$_] - $c0->[$_];
+        my $e1 = $d0->[$_] - $c1->[$_];
+        $e0 *= $e0;
+        $e1 *= $e1;
+        $d2 += ($e0 > $e1 ? $e0 : $e1);
+    }
+    $d2;
 }
 
 sub max_component_index {
     my $self = shift;
     return unless @$self;
-    my $max = $self->[0];
+    my $max = 0;
     my $max_ix = 0;
-    for my $ix (1..$#$self) {
-        if ($self->[$ix] > $max) {
+    for my $ix (0..$#$self) {
+        my $c = CORE::abs($self->[$ix]);
+        if ($c > $max) {
+            $max = $c;
             $max_ix = $ix;
-            $max = $self->[$ix];
         }
     }
     $max_ix;
 }
 
+sub min_component_index {
+    my $self = shift;
+    return unless @$self;
+    my $min = CORE::abs($self->[0]);
+    my $min_ix = 0;
+    for my $ix (1..$#$self) {
+        my $c = CORE::abs($self->[$ix]);
+        if ($c < $min) {
+            $min = $c;
+            $min_ix = $ix
+        }
+    }
+    $min_ix;
+}
+
 sub decompose {
     my ($u, $v) = @_;
     my $p = $u * ($u * $v)/abs2($u);
@@ -352,6 +447,37 @@ sub canonical_base {
     return @base;
 }
 
+sub rotation_base_3d {
+    my $v = shift;
+    @$v == 3 or croak "rotation_base_3d requires a vector with three dimensions";
+    $v = $v->versor;
+    my $n = [0, 0, 0];
+    for (0..2) {
+        if (CORE::abs($v->[$_]) > 0.57) {
+            $n->[($_ + 1) % 3] = 1;
+            $n = $v->decompose($n)->versor;
+            return ($v, $n, $v x $n);
+        }
+    }
+    die "internal error, all the components where smaller than 0.57!";
+}
+
+sub rotate_3d {
+    my $v = shift;
+    my $angle = shift;
+    my $c = cos($angle); my $s = sin($angle);
+    my ($i, $j, $k) = $v->rotation_base_3d;
+    my $rj = $c * $j + $s * $k;
+    my $rk = $c * $k - $s * $j;
+    if (wantarray) {
+        return map { ($_ * $i) * $i + ($_ * $j) * $rj + ($_ * $k) * $rk } @_;
+    }
+    else {
+        my $a = shift;
+        return (($a * $i) * $i + ($a * $j) * $rj + ($a * $k) * $rk);
+    }
+}
+
 sub normal_base { __PACKAGE__->complementary_base(@_) }
 
 sub complementary_base {
@@ -392,6 +518,25 @@ sub complementary_base {
     wantarray ? @base[0..$last] : $base[0];
 }
 
+sub select_in_ball {
+    my $v = shift;
+    my $r = shift;
+    my $r2 = $r * $r;
+    grep $v->dist2($_) <= $r2, @_;
+}
+
+sub select_in_ball_ref2bitmap {
+    my $v = shift;
+    my $r = shift;
+    my $p = shift;
+    my $r2 = $r * $r;
+    my $bm = "\0" x int((@$p + 7) / 8);
+    for my $ix (0..$#$p) {
+        vec($bm, $ix, 1) = 1 if $v->dist2($p->[$ix]) <= $r2;
+    }
+    return $bm;
+}
+
 1;
 __END__
 
@@ -415,8 +560,8 @@ Math::Vector::Real - Real vector arithmetic in Perl
 
 A simple pure perl module to manipulate vectors of any dimension.
 
-The function C<V>, always exported by the module, allows to create new
-vectors:
+The function C<V>, always exported by the module, allows one to create
+new vectors:
 
   my $v = V(0, 1, 3, -1);
 
@@ -536,15 +681,15 @@ inside the grid containing it:
 
   such that ji*wi <= vi <  (ji+1)*wi
 
-=item $max = $v->max
+=item $max = $v->max_component
 
 Returns the maximum of the absolute values of the vector components.
 
-=item $min = $v->min
+=item $min = $v->min_component
 
 Returns the minimum of the absolute values of the vector components.
 
-=item $d2 = $b->abs2
+=item $d2 = $b->norm2
 
 Returns the norm of the vector squared.
 
@@ -558,10 +703,35 @@ Returns the distance between the two vectors squared.
 
 =item ($bottom, $top) = Math::Vector::Real->box($v0, $v1, $v2, ...)
 
-Returns the two corners of a hyper-box containing all the given
-vectors.
+Returns the two corners of the L<axis-aligned minimum bounding
+box|http://en.wikipedia.org/wiki/Minimum_bounding_box#Axis-aligned_minimum_bounding_box>
+(or L<hyperrectangle|http://en.wikipedia.org/wiki/Hyperrectangle>) for
+the given vectors.
 
-=item $d = $v->set($u)
+In scalar context returns the difference between the two corners (the
+box diagonal vector).
+
+=item $p = $v->nearest_in_box($w0, $w1, ...)
+
+Returns the vector nearest to C<$v> from the axis-aligned minimum box
+bounding the given set of vectors.
+
+For instance, given a point C<$v> and an axis-aligned rectangle
+defined by two opposite corners (C<$c0> and C<$c1>), this method can be
+used to find the point nearest to C<$v> from inside the rectangle:
+
+  my $n = $v->nearest_in_box($c0, $c1);
+
+Note that if C<$v> lays inside the box, the nearest point is C<$v>
+itself. Otherwise it will be a point from the box hyper-surface.
+
+=item $d2 = Math::Vector::Real->max_dist2_between_boxes($a0, $a1, $b0, $b1)
+
+Returns the square of the maximum distance between any two points
+belonging respectively to the boxes defined by C<($a0, $a1)> and
+C<($b0, $b1)>.
+
+=item $v->set($u)
 
 Equivalent to C<$v = $u> but without allocating a new object.
 
@@ -569,7 +739,17 @@ Note that this method is destructive.
 
 =item $d = $v->max_component_index
 
-Return the index of the vector component with the maximum size.
+Returns the index of the vector component with the maximum size.
+
+=item $r = $v->first_orthant_reflection
+
+Given the set of vectors formed by C<$v> and all its reflections
+around the axis-aligned hyperplanes, this method returns the one lying
+on the first orthant.
+
+See also
+[http://en.wikipedia.org/wiki/Reflection_%28mathematics%29|reflection]
+and [http://en.wikipedia.org/wiki/Orthant|orthant].
 
 =item ($p, $n) = $v->decompose($u)
 
@@ -589,7 +769,7 @@ wrong result may be generated.
 
 =item @b = $v->normal_base
 
-Returns a set of vectors forming an ortonormal base for the hyperplane
+Returns a set of vectors forming an orthonormal base for the hyperplane
 normal to $v.
 
 In scalar context returns just some unitary vector normal to $v.
@@ -599,32 +779,57 @@ Note that this two expressions are equivalent:
   @b = $v->normal_base;
   @b = Math::Vector::Real->complementary_base($v);
 
+=item ($i, $j, $k) = $v->rotation_base_3d
+
+Given a 3D vector, returns a list of 3 vectors forming an orthonormal
+base where $i has the same direction as the given vector C<$v> and
+C<$k = $i x $j>.
+
+=item @r = $v->rotate_3d($angle, @s)
+
+Returns the vectors C<@u> rotated around the vector C<$v> an
+angle C<$angle> in radians in anticlockwise direction.
+
+See L<http://en.wikipedia.org/wiki/Rotation_operator_(vector_space)>.
+
+=item @s = $center->select_in_ball($radius, $v1, $v2, $v3, ...)
+
+Selects from the list of given vectors those that lay inside the
+n-ball determined by the given radius and center (C<$radius> and
+C<$center> respectively).
+
 =back
 
 =head2 Zero vector handling
 
 Passing the zero vector to some methods (i.e. C<versor>, C<decompose>,
-C<normal_base>, etc.) is not acceptable, in those cases the module
-will croak with a "division by zero" error.
+C<normal_base>, etc.) is not acceptable. In those cases, the module
+will croak with an "Illegal division by zero" error.
 
 C<atan2> is an exceptional case that will return 0 when any of its
 arguments is the zero vector (for consistency with the C<atan2> builtin
 operating over real numbers).
 
-In any case note in practice, rounding errors almost always cause the
-check for the zero vector to fail resulting in numerical
+In any case note that, in practice, rounding errors frequently cause
+the check for the zero vector to fail resulting in numerical
 instabilities.
 
-The correct way to handle it is to introduce in your code checks of
-this kind:
+The correct way to handle this problem is to introduce in your code
+checks of this kind:
 
-  if ($v->abs2 < $epsilon2) {
+  if ($v->norm2 < $epsilon2) {
     croak "$v is too small";
   }
 
 Or even better, reorder the operations to minimize the chance of
 instabilities if the algorithm allows it.
 
+=head2 Math::Vector::Real::XS
+
+The module L<Math::Vector::Real::XS> reimplements most of the methods
+available from this module in XS. When it is installed,
+C<Math::Vector::Real> when automatically load and use it.
+
 =head1 SEE ALSO
 
 L<Math::Vector::Real::Random> extends this module with random vector
@@ -636,9 +841,23 @@ There are other vector manipulation packages in CPAN (L<Math::Vec>,
 L<Math::VectorReal>, L<Math::Vector>), but they can only handle 3
 dimensional vectors.
 
+=head1 SUPPORT
+
+In order to report bugs you can send me and email to the address that
+appears below or use the CPAN RT bug-tracking system available at
+L<http://rt.cpan.org>.
+
+The source for the development version of the module is hosted at
+GitHub: L<https://github.com/salva/p5-Math-Vector-Real>.
+
+=head2 My wishlist
+
+If you like this module and you're feeling generous, take a look at my
+wishlist: L<http://amzn.com/w/1WU1P6IR5QZ42>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009, 2011 by Salvador FandiE<ntilde>o
+Copyright (C) 2009-2012, 2014 by Salvador FandiE<ntilde>o
 (sfandino@yahoo.com)
 
 This library is free software; you can redistribute it and/or modify
